from dataclasses import dataclass, field
from typing import Dict, Union
from datetime import datetime
from wssiham.connection.zeepclient import Zeepclient

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class ModifCoordonneesBancaires(Zeepclient):
    """Ce flux est mis en place pour permettre à des applications gérant le dossier du personnel de pouvoir apporter
    des modifications au niveau de SIHAM. Il donne la possibilité d’ajouter des
    coordonnées bancaires au dossier d’un agent.
    """
    matricule: str
    cle_compte: str
    code_agence: str
    code_banque: str
    date_deb_banque: str
    date_fin_banque: str
    libelle_agence: str
    mode_paiement: str
    num_compte: str
    pays_banque: str

    iban: str = field(default='')
    swift: str = field(default='')
    temoin_validite: str = field(default='')

    def response(self) -> str:
        """ Retourne le statut de la mise à jour "StatutMaj"
        - 1 : OK
        - 2 : KO

        Returns:
            Le nombre d'agent trouvé correspondant aux critres de sélection
        """
        result = -1
        try:
            resp = self.zeep_client.service.ModifCoordonneesBancaires(
                ParamMCB=self.param_mcb())
            if resp:
                result = zeep.helpers.serialize_object(resp)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def param_mcb(self) -> Dict[str, str]:

        parammcb = {'cleCompte': self.cle_compte,
                    'codeAgence': self.code_agence,
                    'codeBanque': self.code_banque,
                    'dateDebBanque': self.ddb_datetime(),
                    'dateFinBanque': self.dfb_datetime(),
                    'IBAN': self.iban,
                    'libelleAgence': self.libelle_agence,
                    'matricule': self.matricule,
                    'modePaiement': self.mode_paiement,
                    'numCompte': self.num_compte,
                    'paysBanque': self.pays_banque,
                    'SWIFT': self.swift,
                    'temoinValidite': self.temoin_validite}

        return parammcb

    def ddb_datetime(self) -> Union[datetime, str]:
        if self.date_deb_banque:
            return datetime.strptime(self.date_deb_banque, '%d/%m/%Y')
        return self.date_deb_banque

    def dfb_datetime(self) -> Union[datetime, str]:
        if self.date_fin_banque:
            return datetime.strptime(self.date_fin_banque, '%d/%m/%Y')
        return self.date_fin_banque
