from dataclasses import dataclass, field
from typing import List, Any, Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.dossier_agent.recupDonneesDeServiceResponse import InformationService
from datetime import datetime

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupDonneesDeService(Zeepclient):
    """ L’objectif de ce flux est de consulter les informations de décharge de service de la population SIHAM
    qui rentre dans les critères
    """
    matricule: str = field(default='')

    code_etablissement: str = field(default='')                 # exemple : universite de la reunion = 9740478B
    date_observation: datetime = field(default='')         # format YYYY-MM-DD-HH.MM.SS
    date_fin_observation: datetime = field(default='')     # format YYYY-MM-DD-HH.MM.SS

    liste_matricule: Any = field(init=False, default=None)

    def __post_init__(self):
        super().__post_init__()
        self.__initialisation_object_liste()

    def response(self) -> List[Optional[InformationService]]:
        """ Récupere les données de décharge de service de l'agent

        Returns:
            Liste contenant les informations administratives de l'agent recherché
                - Matricule de l’agent
                - Code UAI de l’établissement
                - Libellé court de l’établissement
                - Libellé long de l’établissement
                - Type de décharge
                - Libellé long du type de décharge
                - Nombre d’heures

        """
        result = []
        try:
            resp = \
                self.zeep_client.service.RecupDonneesDeService(ParamDonneesService=
                                                              {'codeEtablissement': self.code_etablissement,
                                                               'dateFinObservation': self.date_fin_observation,
                                                               'dateObservation': self.date_observation,
                                                               'listeMatricules': self.liste_matricule})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = InformationService(**tmp_data[0])
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice RecupDonneesDeService

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice RecupDonneesDeService

        """
        t_lmatricule = self.zeep_client.get_type('ns0:matricule')

        self.liste_matricule = t_lmatricule(self.matricule)

    # def do_datetime(self) -> Union[datetime, str]:
    #     if self.date_observation:
    #         return datetime.strptime(self.date_observation, '%d/%m/%Y')
    #     return self.date_observation
    #
    # def dfo_datetime(self) -> Union[datetime, str]:
    #     if self.date_fin_observation:
    #         return datetime.strptime(self.date_observation, '%d/%m/%Y')
    #     return self.date_fin_observation
