from dataclasses import dataclass, field
from typing import List, Any, Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.dossier_agent.recupProchainEchelonReponse import InformationProchainEchelon
from datetime import datetime

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupProchainEchelon(Zeepclient):
    """SIHAM permet de simuler l’avancement d’échelon des agents. Cette information peut être exposée, via ce flux,
    aux applications externes. Il est possible de consulter les données d’avancement d’échelon pour un ou plusieurs
    agents en fonction d’une date d’observation.
    """
    matricule: str = field(default='')

    code_etablissement: str = field(default='')                 # exemple : universite de la reunion = 9740478B
    date_observation: datetime = field(default='')       # format YYYY-MM-DD-HH.MM.SS
    date_fin_observation: datetime = field(default='')               # format YYYY-MM-DD-HH.MM.SS

    liste_matricule: Any = field(init=False, default=None)

    def __post_init__(self):
        super().__post_init__()
        self.__initialisation_object_liste()

    def response(self) -> List[Optional[InformationProchainEchelon]]:
        """ Récupere les données d'avancement d'échelon
        """
        result = []
        try:
            resp = \
                self.zeep_client.service.RecupProchainEchelon(ParamListAgent=
                                                              {'codeEtablissement': self.code_etablissement,
                                                               'dateFinObservation': self.date_fin_observation,
                                                               'dateObservation': self.date_observation,
                                                               'listeMatricules': self.liste_matricule})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = InformationProchainEchelon(prochainEchelon=tmp_data[0])
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice RecupDonneesPersonnelles

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice RecupDonneesPersonnelles

        """
        t_lmatricule = self.zeep_client.get_type('ns0:matricule')

        self.liste_matricule = t_lmatricule(self.matricule)

