from dataclasses import dataclass, field
from typing import List, Any, Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.dossier_agent.recupTableauAvancementResponse import InformationTableauAvancement
from datetime import datetime

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupTableauAvancement(Zeepclient):
    """Cette opération permet d’envoyer les informations concernant les propositions d’avancement d’un ou
    plusieurs agents, pour les avancements de grade, les promotions internes et les reclassements.
    """
    matricule: str = field(default='')

    code_etablissement: str = field(default='')                 # exemple : universite de la reunion = 9740478B
    date_observation: datetime = field(default='')       # format YYYY-MM-DD-HH.MM.SS
    code_type_avancement: str = field(default='')

    liste_matricule: Any = field(init=False, default=None)

    def __post_init__(self):
        super().__post_init__()
        self.__initialisation_object_liste()

    def response(self) -> List[Optional[InformationTableauAvancement]]:
        """ Récupere les données d'avancement d'échelon
        """
        result = []
        try:
            resp = \
                self.zeep_client.service.RecupTableauAvancement(ParamTableauAvancement=
                                                                {'codeEtablissement': self.code_etablissement,
                                                                 'codeTypeAvancement': self.code_type_avancement,
                                                                 'dateObservation': self.date_observation,
                                                                 'listeMatricules': self.liste_matricule})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = InformationTableauAvancement(**tmp_data[0])
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice RecupTableauAvancement

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice RecupTableauAvancement

        """
        t_lmatricule = self.zeep_client.get_type('ns0:matricule')

        self.liste_matricule = t_lmatricule(self.matricule)


