from dataclasses import dataclass, field
from typing import List, Any, Union, Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.dossier_agent_date.recupDonneesAgentsResponse import InformationPersonnelle
from collections import OrderedDict
from datetime import datetime

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupDonneesAgents(Zeepclient):
    """ Ce Web Service permet d’extraire l’exhaustivité des attributs de l’agent en vigueur ou non à la
    date d’observation.
    """
    matricule: str                                          # matricule de l'agent recherché
    date_observation: str = field(default='')               # format YYYY-MM-DD-HH.MM.SS
    liste_matricule: Any = field(init=False, default=None)

    def __post_init__(self):
        super().__post_init__()
        self.__initialisation_object_liste()

    def response(self) -> List[Optional[InformationPersonnelle]]:
        """ Récupere l’exhaustivité des attributs de l’agent en vigueur ou non à la date d’obervation

        Returns:
            Liste contenant les données personnelles et professionnelles de l'agent


        """
        result = []
        try:
            resp = \
                self.zeep_client.service.RecupDonneesAgents(ParamAgentDateWS={'dateObservation': self.do_datetime(),
                                                                              'listeMatricules': self.matricule})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = InformationPersonnelle(**tmp_data[0])
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice RecupDonneesAgents

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice RecupDonneesAgents

        """
        t_lmatricule = self.zeep_client.get_type('ns0:matricule')
        self.liste_matricule = t_lmatricule(self.matricule)

    def do_datetime(self) -> Union[datetime, str]:
        if self.date_observation:
            return datetime.strptime(self.date_observation, '%d/%m/%Y')
        return self.date_observation
