from dataclasses import dataclass, field
from typing import List, Any, Union
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.dossier_parametrage.recupStructuresResponse import UniteOrganisationnelle

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupStructures(Zeepclient):
    """ Ce flux a pour objectif d’envoyer toutes les informations pertinentes sur les unités
    organisationnelles dans SIHAM.
    """
    code_uo: str = field(default='')                  # 3 caractère CDSTCO ZD00 (exemple: HJB = grade)
    type_uo: str = field(default='')
    code_administration: str = field(init=False, default='')    # 3 caractère (exemple: 'RUN' pour Université Réunion
    date_observation: str = field(default='')                   # format dd/mm/YYYY

    liste_uo: Any = field(init=False, default=None)

    def __post_init__(self):
        super().__post_init__()
        if self.code_uo is None and self.structure_uo is None and \
                self.type_uo is None and \
                self.date_observation is None:
            raise TypeError("Valeur requise pour au moins un champ.")

        self.__initialisation_object_liste()

    def response(self) -> Union[UniteOrganisationnelle, List]:
        """ Récuperer les données en vigueur à une date donnée pour une unité organisationnel

        Returns:
            Liste contenant les informations sur les données d'une ou des unités organisationnels

        """
        result = []
        try:
            resp = \
                self.zeep_client.service.RecupStructures(ParamStructure=
                                                         {'dateObservation': self.date_observation,
                                                          'codeAdministration': self.code_administration,
                                                          'listeUO': self.liste_uo})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result.append(UniteOrganisationnelle(**tmp_data[0]))
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice RecupNomenclaturesRH

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice RecupNomenclaturesRH

        """
        t_luo = self.zeep_client.get_type('ns0:listeUO')

        self.liste_uo = t_luo(codeUO=self.code_uo, typeUO=self.type_uo)


