from dataclasses import dataclass, field
from typing import List, Union
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_agents.genericAgentsReponse import AgentDataEcheanceList

from datetime import datetime
import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupAgentsEcheance(Zeepclient):
    """ ce Web Service retourne les matricules des agents dont l’un des attributs échangés,
    gérés par date, est entré en vigueur ou arrive à échéance entre les deux dates données en paramètre.
    """

    date_debut_observation: str = field(default='')               # format dd/mm/YYYY
    date_fin_observation: str = field(default='')
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Union[AgentDataEcheanceList, List]:
        """ Retourne les codes des structures dont l’un des attributs échangés entre en vigueur ou
         arrive à échéance entre les deux dates données


        Returns:
            Liste contenant les informations concernant le matricule, les témoins et le login Ldap de l'agent.
            (matricule, témoin état de l'agent, témoin retraite, UID)

        """
        result = []
        try:
            resp = self.zeep_client.service.RecupAgentsEcheance(
                ParamAgentEcheance={'dateDebutObservation': datetime.strptime(self.date_debut_observation, '%d/%m/%Y'),
                                    'dateFinObservation': self.dfo_datetime()})

            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = AgentDataEcheanceList(agent_data_echeance=tmp_data)
                self.found = (True, len([ele for ele in result if isinstance(ele, dict)]))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def dfo_datetime(self) -> Union[datetime, str]:
        if self.date_fin_observation:
            return datetime.strptime(self.date_fin_observation, '%d/%m/%Y')
        return self.date_fin_observation
