from datetime import datetime
from dataclasses import dataclass, field
from typing import List, Union, Dict, Any
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_agents.genericAgentsReponse import AgentDataLA

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupListeAgents(Zeepclient):
    """ Le flux permet d’effectuer une recherche dans SIHAM selon plusieurs critères : recherche par nom,
    par affectation, …etc.
    """

    code_etablissement: str = field(default='')                 # code de l'établissement (exemple : 9740478B)
    date_observation: str = field(default='')                   # format dd/mm/YYYY
    nom_patronymique: str = field(default='')
    nom_usuel: str = field(default='')
    prenom: str = field(default='')
    numero_insee: str = field(default='')

    # listeAbsencesConges
    liste_absences_conges: Any = field(init=False, default=None)
    code_absence_conges: str = field(default='')

    # listeAffectationUO
    liste_affectation_uo: Any = field(init=False, default= None)
    code_uo: str = field(default='')
    temoin_uo_fille: str = field(default='')
    type_lien_affectation_uo: str = field(default='')

    # listeCorps
    liste_corps: Any = field(init=False, default=None)
    code_corps: str = field(default='')

    # listeFonction
    liste_fonction: Any = field(init=False, default=None)
    code_fonction: str = field(default='')
    code_type_fonction: str = field(default='')

    # listeGrades
    liste_grade: Any = field(init=False, default=None)
    code_grade: str = field(default='')

    # listeGroupeHierarchique
    liste_groupe_hierarchique: Any = field(init=False, default=None)
    code_groupe_hierarchique: str = field(default='')

    # listeModaliteService
    liste_modalite_service: Any = field(init=False, default=None)
    code_modalite_service: str = field(default='')

    # ListePIP
    liste_pip: Any = field(init=False, default=None)
    code_pip: str = field(default='')

    # listePositionsAdministratives
    liste_positions_administratives: Any = field(init=False, default=None)
    code_position_administrative: str = field(default='')

    # listeQualitesStatutaires
    liste_qualites_statutaires: Any = field(init=False, default=None)
    code_qualite_statutaire: str = field(default='')

    # listeResponsHierarchiqUO
    liste_respons_hierarchiq_uo: Any = field(init=False, default=None)
    code_uo_responsable: str = field(default='')

    # listeTypeContrat
    liste_type_contrat: Any = field(init=False, default=None)
    code_type_contrat: str = field(default='')
    mode_gest: str = field(default='')

    # specialitesniv1Agent
    specialites_niv1_agent: Any = field(init=False, default=None)
    code_specialite_niv_1_agt: str = field(default='')
    code_type_specialite_agent: str = field(default='')
    temoin_heritage: str = field(default='')
    tem_enseignant_chercheur: str = field(default='')                   # "1" = Enseignant/chercheur; "0" BIATSS

    # "A" = situation en vigueur à la date d'observation
    # "P" = situation en prévision par rapport à la date d'observation
    # "I" = situation clôturée par rapport à la date d'observation
    tem_etat: str = field(default='')

    temoin_valide: str = field(default='')

    date_fin_observation: str = field(default='')
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def __post_init__(self):
        super().__post_init__()

        self.__initialisation_object_liste()

    def __initialisation_object_liste(self):

        t_lauo = self.zeep_client.get_type('ns0:listeAffectationUO')
        t_lac = self.zeep_client.get_type('ns0:listeAbsencesConges')
        t_lcorps = self.zeep_client.get_type('ns0:listeCorps')
        t_lfonction = self.zeep_client.get_type('ns0:listeFonction')
        t_lgrades = self.zeep_client.get_type('ns0:listeGrades')
        t_lgh = self.zeep_client.get_type('ns0:listeGroupeHierarchique')
        t_lms = self.zeep_client.get_type('ns0:listeModaliteService')
        t_lpip = self.zeep_client.get_type('ns0:listePIP')
        t_lpa = self.zeep_client.get_type('ns0:listePositionsAdministratives')
        t_lqs = self.zeep_client.get_type('ns0:listeQualitesStatutaires')
        t_lrhuo = self.zeep_client.get_type('ns0:listeResponsHierarchiqUO')
        t_ltc = self.zeep_client.get_type('ns0:listeTypeContrat')
        t_sna = self.zeep_client.get_type('ns0:listeSpecialitesniv1Agent')

        self.liste_affectation_uo = t_lauo(codeUO=self.code_uo, temoinUOFille=self.temoin_uo_fille,
                                           typeLienAffectationUO=self.type_lien_affectation_uo)

        self.liste_absences_conges = t_lac(codeAbsenceConges=self.code_absence_conges)

        self.liste_corps = t_lcorps(codeCorps=self.code_corps)

        self.liste_fonction = t_lfonction(codeFonction=self.code_fonction, codeTypefonction=self.code_type_fonction)

        self.liste_grade = t_lgrades(codeGrade=self.code_grade)

        self.liste_groupe_hierarchique = t_lgh(codeGroupeHierarchique=self.code_groupe_hierarchique)

        self.liste_modalite_service = t_lms(codeModaliteService=self.code_modalite_service)

        self.liste_pip = t_lpip(codePIP=self.code_pip)

        self.liste_positions_administratives = t_lpa(codePositionAdministrative=self.code_position_administrative)

        self.liste_qualites_statutaires = t_lqs(codeQualiteStatutaire=self.code_qualite_statutaire)

        self.liste_respons_hierarchiq_uo = t_lrhuo(codeUOresponsable=self.code_uo_responsable)

        self.liste_type_contrat = t_ltc(codeTypeContrat=self.code_type_contrat, modeGest=self.mode_gest)

        self.specialites_niv1_agent = t_sna(codeSpecialiteNiv1Agt=self.code_specialite_niv_1_agt,
                                            codeTypeSpecialiteAgent=self.code_type_specialite_agent,
                                            temoinHeritage=self.temoin_heritage)

    def response(self) -> Union[AgentDataLA, List]:
        """ Récupère les données administratives de l'agent

        Returns:
            Liste contenant les informations d'agents suivantes :
                - Code UAI de l’établissement
                - Matricule de l’agent
                - Numéro du dossier dans Harpege
                - Code de la qualité
                - Libellé de la civilité
                - Nom usuel
                - Prénom
                - Nom patronymique
                - Date de naissance
                - Numéro INSEE définitif
                - Numéro INSEE provisoire
                - Code poste de l’affectation hiérarchique dans l’établissement
                - Libellé long du code poste de l’affectation hiérarchique dans l’établissement
                - Code de l’emploi
                - Libellé long de l’emploi de l’affectation
                - Code de l’unité organisationnelle
                - Libellé long de l’unité organisationnelle
                - Date de début d’affectation hiérarchique
                - Date de fin d’affectation hiérarchique
                - Quotité d’affectation hiérarchique
                - Numéro de téléphone fixe professionnel
                - Numéro de portable professionnel
                - Numéro de fax professionnel
                - Adresse mail professionnelle
        """
        result = []
        try:
            resp = \
                self.zeep_client.service.recupListeAgents(ParamRecupListeAgents=self.param_recuplisteagents())
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = AgentDataLA(personal_data_la=tmp_data)
                self.found = (True, len([ele for ele in result if isinstance(ele, dict)]))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def param_recuplisteagents(self) -> Dict[str, str]:
        """ Paramètre pour l'utilisation du webservice rechercheAgent

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice rechercheAgent

        """

        paramrla = {'codeEtablissement': self.code_etablissement,
                    'dateObservation': self.do_datetime(),
                    'nomPatronymique': self.nom_patronymique,
                    'nomUsuel': self.nom_usuel,
                    'numeroInsee': self.numero_insee,
                    'prenom': self.prenom,
                    'temEnseignantChercheur': self.tem_enseignant_chercheur,
                    'temEtat': self.tem_etat,
                    'temoinValide': self.temoin_valide,
                    'listeAbsencesConges': self.liste_absences_conges,
                    'listeAffectationUO': self.liste_affectation_uo,
                    'listeCorps': self.liste_corps,
                    'listeFonction': self.liste_fonction,
                    'listeGrades': self.liste_grade,
                    'listeGroupeHierarchique': self.liste_groupe_hierarchique,
                    'listeModaliteService': self.liste_modalite_service,
                    'listePIP': self.liste_pip,
                    'listePositionsAdministratives': self.liste_positions_administratives,
                    'listeQualitesStatutaires': self.liste_qualites_statutaires,
                    'listeResponsHierarchiqUO': self.liste_respons_hierarchiq_uo,
                    'listeTypeContrat': self.liste_type_contrat,
                    'specialitesniv1Agent': self.specialites_niv1_agent}

        return paramrla

    def do_datetime(self) -> Union[datetime, str]:
        if self.date_observation:
            return datetime.strptime(self.date_observation, '%d/%m/%Y')
        return self.date_observation
