from dataclasses import dataclass, field
from typing import Dict, List, Union
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.nomenclatures_pec.recupNomenclaturesPecResponse import NomenclaturePecList

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupNomenclaturesPec(Zeepclient):
    """
    L’objectif de ce flux est de vérifier la présence d’un agent et, s’il existe de renvoyer sa situation.
    """

    code_repertoire: str                                        # 3 caractère CDSTCO ZD00 (exemple: HJB = grade)
    date_observation: str = field(default='')                   # format dd/mm/YYYY
    code_administration: str = field(default='')                # 3 caractère (exemple: 'RUN' pour Université Réunion
    code_nomenclatures: str = field(default='')                 # 3 caractère CDCODE ZD00

    def response(self) -> Union[NomenclaturePecList, List]:
        """ Récuperer les référentiels dans SIHAM

        Returns:
            Liste contenant les informations concernant les valeurs présentes dans le répertoire
            (code, libellé long, libellé court, date de début et de fin de validité)

        """
        result = []
        try:
            resp = \
                self.zeep_client.service.RecupNomenclaturesPEC(DateObservation=self.date_observation,
                                                               CodeAdministration=self.code_administration,
                                                               listeNomenclatures=self.param_liste_nomenclatures())
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = NomenclaturePecList(nomenclaturesPec=tmp_data)
                logger.debug(f"Liste des paramètres utilisés pour ParamRechercheAgent \n"
                             f"{self.param_liste_nomenclatures()}")
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def param_liste_nomenclatures(self) -> Dict[str, str]:
        """ Paramètre pour l'utilisation du webservice rechercheAgent

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice rechercheAgent

        """
        paramra = {'codeNomenclature': self.code_nomenclatures,
                   'codeRepertoire': self.code_repertoire}

        return paramra

