from dataclasses import dataclass, field
from typing import Dict, Union
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.pec.genericPecResponse import GenericPecResponse

import zeep
import logging
import datetime

logger = logging.getLogger(__name__)


@dataclass
class ModificationDatePEC(Zeepclient):
    """L’objectif de ce flux est d'ajouter une administration d’origine au dossier de l’agent.
    """

    matricule: str
    date_debut_adm_orig: datetime.datetime
    categorie_statutaire: str = field(default='')
    code_corps_origine: str = field(default='')
    code_echelon_origine: str = field(default='')
    code_grade_origine: str = field(default='')
    code_organisme_origine: str = field(default='')
    date_entree_corps: datetime.datetime = field(default='')
    date_entree_grade: datetime.datetime = field(default='')
    date_fin_adm_orig: datetime.datetime = field(default='')
    lib_long_autre_organisme: str = field(default='')
    qualite_statutaire_origine: str = field(default='')
    statut_origine: str = field(default='')
    type_organisme: str = field(default='')

    def response(self) -> Union[GenericPecResponse, Dict]:
        """Ce web service permet l’ajout d’une administration d’origine au dossier de l’agent.
        L’appel à ce Web Service se fait par interrogation sur une liste de paramètres
        (matricule, date de début de l’administration d’origine, corps d’origine, etc.).

        Returns:
            Le tableau de résultat de l’annulation contient les informations suivantes:
                o	Matricule de l’agent,
                o	Liste des anomalies rencontrées
                o	Statut de l’opération (Qui indique si l’annulation a bien été effectuée).

        """
        result = {}
        try:
            resp = self.zeep_client.service.AjoutAdministrationOrigine(
                ParamAAO={'categorieStatutaire': self.categorie_statutaire,
                          'codeCorpsOrigine': self.code_corps_origine,
                          'codeEchelonOrigine': self.code_echelon_origine,
                          'codeGradeOrigine': self.code_grade_origine,
                          'codeOrganismeOrigine': self.code_organisme_origine,
                          'dateDebutAdmOrig': self.date_debut_adm_orig,
                          'dateEntreeCorps': self.date_entree_corps,
                          'dateEntreeGrade': self.date_entree_grade,
                          'dateFinAdmOrig': self.date_fin_adm_orig,
                          'libLongAutreOrganisme': self.lib_long_autre_organisme,
                          'matricule': self.matricule,
                          'qualiteStatutaireOrigine': self.qualite_statutaire_origine,
                          'statutOrigine': self.statut_origine,
                          'typeOrganisme': self.type_organisme}
            )
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = GenericPecResponse(**tmp_data)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result
