from dataclasses import dataclass, field
from typing import Dict, Union
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.pec.genericPecResponse import GenericPecResponse

import zeep
import logging
import datetime

logger = logging.getLogger(__name__)


@dataclass
class ModifAffectationSecondaire(Zeepclient):
    """L’objectif de ce web-service est de permettre de modifier les occurrences d’affectation secondaire d’un agent.
    """

    pourcentage_affectation: int
    code_uo_affect_adresse: str = field(default='')
    type_action: str = field(default='')
    type_rattachement: str = field(default='')
    matricule: str = field(default='')
    date_debut: datetime.datetime = field(default='')
    date_fin: datetime.datetime = field(default='')
    iban: str = field(default='')
    swift: str = field(default='')
    temoin_validite: str = field(default='')

    def response(self) -> Union[GenericPecResponse, Dict]:
        """L’appel à ce Web Service se fait par interrogation sur une liste de paramètres
        (matricule, date de début et de fin de l’affectation secondaire, type de rattachement, etc.).

        Returns:
            -	Le tableau de résultat de l’annulation contient les informations suivantes:
                o	Matricule de l’agent,
                o	Liste des anomalies rencontrées
                o	Statut de l’opération (Qui indique si l’annulation a bien été effectuée).
        """
        try:
            resp = self.zeep_client.service.ModifAffectationSecondaire(
                ParamMAS={'codeUOAffectAdresse': self.code_uo_affect_adresse,
                          'dateDebut': self.date_debut,
                          'dateFin': self.date_fin,
                          'matricule': self.matricule,
                          'pourcentageAffectation': self.pourcentage_affectation,
                          'temoinValidite': self.temoin_validite,
                          'typeAction': self.type_action,
                          'typeRattachement': self.type_rattachement}
            )
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = GenericPecResponse(**tmp_data)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result
