from dataclasses import dataclass, field
from typing import Dict, Union, Any
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.pec.genericPecResponse import GenericPecResponse

import datetime
import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class PriseEnChargeAgent(Zeepclient):
    """L’objectif de ce flux est de prendre en charge des agents.
    """
    nom_usuel: str
    prenom: str
    categorie_entree: str                   # 'ACTIVE'
    civilite: str                           # '1' = M., '2' = Mme, '3' = Mlle
    code_administration: str                # 'RUN'
    code_etablissement: str                 # '9740478B'
    sexe: str                               # '1' = Masculin, '2' = Feminin
    date_embauche: datetime.datetime
    date_naissance: datetime.datetime
    departement_naissance: str = field(default='')
    emploi: str = field(default='')

    liste_carriere: Any = field(init=False, default=None)
    chevron_carriere: str = field(default='')
    date_effet_carriere: datetime.datetime = field(default='')
    date_fin: datetime.datetime = field(default='')
    echelon: str = field(default='')
    grade: str = field(default='')
    indice_brut: str = field(default='')
    indice_majore: str = field(default='')
    mode_acces_echelon: str = field(default='')
    mode_acces_grade: str = field(default='')
    numCarriere: str = field(default='')
    numord: str = field(default='')
    qualite_statutaire: str = field(default='')
    tem_rem_acessoire: str = field(default='')
    temoin_validite: str = field(default='')

    liste_contrats: Any = field(init=False, default=None)
    chevron_contrat: str = field(default='')
    commentaire_contrat: str = field(default='')
    date_debut_contrat: datetime.datetime = field(default='')
    date_fin_contrat: datetime.datetime = field(default='')
    debut_periode_essai: datetime.datetime = field(default='')
    devise_remuneration_contrat: str = field(default='')
    fin_periode_essai: str = field(default='')
    grade_tg: str = field(default='')
    hors_echelle_lettre: str = field(default='')
    indice_remu: str = field(default='')
    mode_de_gestion: str = field(default='')
    mode_remuneration: str = field(default='')
    nature_contrat: str = field(default='')
    nb_heures_contrat: str = field(default='')
    pourcentage: int = field(default=0)
    remuneration_contrat: int = field(default=0)
    taux_horaires: float = field(default=0)
    # temoin_validite
    type_contrat: str = field(default='')
    type_lien_juridique: str = field(default='')

    liste_coordonnees_postales: Any = field(init=False, default=None)
    bis_ter: str = field(default='')
    bureau_distributeur: str = field(default='')
    insefr: str = field(default='')
    code_pays: str = field(default='')
    code_postal: str = field(default='')
    commune: str = field(default='')
    complement_adresse: str = field(default='')
    debut_adresse: str = field(default='')
    fin_adresse: str = field(default='')
    nature_voie: str = field(default='')
    nom_voie: str = field(default='')
    num_adresse: str = field(default='')
    tem_adresse_principale: str = field(default='')
    # temoin_validite
    type_adresse: str = field(default='')

    liste_coordonnees_bancaires: Any = field(init=False, default=None)
    cle_compte: str = field(default='')
    code_agence: str = field(default='')
    code_banque: str = field(default='')
    date_deb_banque: datetime.datetime = field(default='')
    date_fin_banque: datetime.datetime = field(default='')
    iban: str = field(default='')
    libelle_agent: str = field(default='')
    mode_paiement: str = field(default='')
    num_compte: str = field(default='')
    pays_banque: str = field(default='')
    swift: str = field(default='')
    # temoin_validite

    liste_modalites_services: Any = field(init=False, default=None)
    date_effet_modalite: datetime.datetime = field(default='')
    modalite: str = field(default='')
    # temoin_validite

    liste_nationalites: Any = field(init=False, default=None)
    mode_acquisition_nationalite: str = field(default='')
    nationalite: str = field(default='')
    tem_principale: str = field(default='')

    liste_numeros_telephone_fax: Any = field(init=False, default=None)
    date_debut_tel: datetime.datetime = field(default='')
    numero: str = field(default='')
    type_numero: str = field(default='')

    liste_positions: Any = field(init=False, default=None)
    date_effet_position: datetime.datetime = field(default='')
    date_fin_prevue: datetime.datetime = field(default='')
    date_fin_reelle: datetime.datetime = field(default='')
    position: str = field(default='')
    # temoin_validite

    # Important
    liste_statuts: Any = field(init=False, default=None)
    date_effet_statut: datetime.datetime = field(default='')
    statut: str = field(default='')
    # temoin_validite

    liste_situations_familiales: Any = field(init=False, default=None)
    date_effet_situ_fam: datetime.datetime = field(default='')
    situ_fam: str = field(default='')
    # temoin_validite

    motif_entree: str = field(default='')
    nom_patronymique: str = field(default='')
    numero_insee: str = field(default='')
    numero_insee_provisoire: str = field(default='')
    pays_naissance: str = field(default='')
    poste: str = field(default='')
    # temoin_validite
    uo: str = field(default='')
    ville_naissance: str = field(default='')

    def __post_init__(self):
        super().__post_init__()
        self.__initialisation_object_liste()

    def response(self) -> Union[GenericPecResponse, Dict]:
        """ L’objectif de ce flux est de prendre en charge des agents.
        L’appel à ce Web Service se fait par interrogation sur une liste de paramètres (état civil, date de naissance,
        affectations, carrière, statuts, position etc.).
        Afin de tracer dans le dossier agent, l’information indiquant que le dossier ait bien été créé via web-service,
        la méthode lors de la création, crée une occurrence de Matricules précédents (ZYU5) avec les valeurs suivantes :
            -	IDEMPR (matricule précédent) : Valorisé par ‘WS_PEC’.
            -	DTCHNG (date de changement) : Valorisé par la date du jour


        Returns:
            Liste orderedDict contenant les structures de données suivante:
                - Matricule : Matricule de l’agent
                - Statut : Statut de la PEC
                - Anomalie Web Service : Contrôle du Web Service

        Notes:
            Exemple de "response" d'une prise en charge valide :
                - OrderedDict([('listeAnomaliesWebServices', [OrderedDict([('anomalie',
                "Warnning Trt =AE03BDCA Info=3B  03BD08 Le taux d'affectation sera initialisé à 100%. ")]),
                OrderedDict([('anomalie',
                'Warnning Trt =nullnull Info=3B TEMHER A Anomalie sur mouvement Rubrique alimentée par défaut ')]),
                OrderedDict([('anomalie',
                "Warnning Trt =YA0F91UU Info=3B  0F9003 Aucun code de gestion n'est paramétré pour cette affectation
                avec le statut de l'agent, veuillez modifier manuellement le code de gestion. ")]),
                OrderedDict([('anomalie', "Warnning Trt =YA0F91UU Info=FL  0F9003 Aucun code de gestion n'est paramétré
                pour cette affectation avec le statut de l'agent,
                veuillez modifier manuellement le code de gestion. ")]),
                OrderedDict([('anomalie',
                'Warnning Trt =nullnull Info=TL TYJBSR A Anomalie sur mouvement Rubrique alimentée par défaut ')]),
                OrderedDict([('anomalie',
                'Warnning Trt =JA0051NA Info=  005XXX Nom patronymique alimenté par défaut. ZY05NOMPAT')]),
                OrderedDict([('anomalie', "Warnning Trt =YA0P9A Info=3B  03B205 Il faut saisir soit le poste,
                soit l'emploi & l'unité organisationnelle. ")]),
                OrderedDict([('anomalie',
                "Warnning Trt =JX0ANEV2 Info=10  9EVE01 Evénement T-10-AGT - Naissance de l'agent ")]),
                OrderedDict([('anomalie', "Warnning Trt =JX0ANEV2 Info=10  9REG01 Règle de gestion  R-DONSIT-01 -
                La situation familiale de l'agent doit être renseignée. ")]),
                OrderedDict([('anomalie', 'Warnning Trt =JX0ANEV2 Info=10  9REG01 Règle de gestion  R-DONNIR-01 -
                Un NIR ( provisoire ou définitif ) doit obligatoirement être renseigné. ')]),
                OrderedDict([('anomalie', "Warnning Trt =UA0192 Info=19  019202 La date d'entrée dans le groupe est
                initialisée à la date d'embauche. ")]),
                OrderedDict([('anomalie', "Warnning Trt =JA0192NK Info=19  0FK101 La date légale de départ en retraite
                n'est pas calculée lors de la prise en charge. Un mouvement sur la carrière sera nécessaire ")]),
                OrderedDict([('anomalie',
                'Warnning Trt =YA0YP1 Info=YP  0YP1Y2 Population inexistente : (2023-02-13;HB015;  )')]),
                OrderedDict([('anomalie', 'Warnning Trt =AS05G1TL Info=  05G106 Durée théorique de la semaine supérieure
                à celle contractuelle en date du : 2023-02-13')])]),
                ('matricule', 'RUN000101770'), ('statut', 'MAJ OK')])


        """
        result = {}
        try:
            resp = self.zeep_client.service.PriseEnChargeAgent(
                ParamPEC={'categorieEntree': self.categorie_entree, 'civilite': self.civilite,
                          'codeAdministration': self.code_administration,
                          'codeEtablissement': self.code_etablissement,
                          'dateEmbauche': self.date_embauche,
                          'dateNaissance': self.date_naissance,
                          'departementNaissance': self.departement_naissance,
                          'emploi': self.emploi,
                          'listeCarriere': self.liste_carriere,
                          'listeContrats': self.liste_contrats,
                          'listeCoordonneesPostales': self.liste_coordonnees_postales,
                          'listeModalitesServices': self.liste_modalites_services,
                          'listeNationalites': self.liste_nationalites,
                          'listeNumerosTelephoneFax': self.liste_numeros_telephone_fax,
                          'listePositions': self.liste_positions,
                          'listeStatuts': self.liste_statuts,
                          'listesituationsfamiliales': self.liste_situations_familiales,
                          'motifEntree': self.motif_entree, 'nomPatronymique': self.nom_patronymique,
                          'nomUsuel': self.nom_usuel, 'numeroInsee': self.numero_insee,
                          'numeroInseeProvisoire': self.numero_insee_provisoire,
                          'paysNaissance': self.pays_naissance, 'poste': self.poste,
                          'prenom': self.prenom, 'sexe': self.sexe, 'temoinValidite': self.temoin_validite,
                          'UO': self.uo, 'villeNaissance': self.ville_naissance
                          }
            )
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = GenericPecResponse(**tmp_data)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice PriseEnChargeAgent

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice PriseEnChargeAgent

        """
        t_lcarriere = self.zeep_client.get_type('ns0:listeCarriere')
        t_lcontrats = self.zeep_client.get_type('ns0:listeContrats')
        t_lcoordonneespostales = self.zeep_client.get_type('ns0:listeCoordonneesPostales')
        t_lcoordonneesbancaires = self.zeep_client.get_type('ns0:listeCoordonneesbancaires')
        t_lmodalitesservice = self.zeep_client.get_type('ns0:listeModalitesServices')
        t_lnationalites = self.zeep_client.get_type('ns0:listeNationalites')
        t_lnumerotelfax = self.zeep_client.get_type('ns0:listeNumerosTelephoneFax')
        t_lpostions = self.zeep_client.get_type('ns0:listePositions')
        t_lstatus = self.zeep_client.get_type('ns0:listeStatuts')
        t_lsituationsfamiliales = self.zeep_client.get_type('ns0:listeSituationsfamiliales')

        self.liste_carriere = t_lcarriere(chevron=self.chevron_carriere,
                                          dateEffetCarriere= self.date_effet_carriere,
                                          dateFin=self.date_fin,
                                          echelon=self.echelon, grade=self.grade, indiceBrut=self.indice_brut,
                                          indiceMajore=self.indice_majore, modeAccesEchelon=self.mode_acces_echelon,
                                          modeAccesGrade=self.mode_acces_grade, numCarriere=self.numCarriere,
                                          numord=self.numord, qualiteStatutaire=self.qualite_statutaire,
                                          temRemAcessoire=self.tem_rem_acessoire, temoinValidite=self.temoin_validite)

        self.liste_contrats = t_lcontrats(chevron=self.chevron_contrat, commentaireContrat=self.commentaire_contrat,
                                          dateDebutContrat=self.date_debut_contrat,
                                          dateFinContrat=self.date_fin_contrat,
                                          debutPeriodeEssai=self.debut_periode_essai,
                                          deviseRemunerationContrat=self.devise_remuneration_contrat,
                                          finPeriodeEssai=self.fin_periode_essai,
                                          gradeTG=self.grade_tg, horsEchelleLettre=self.hors_echelle_lettre,
                                          indiceRemu=self.indice_remu, modeDeGestion=self.mode_de_gestion,
                                          modeRemuneration=self.mode_remuneration, natureContrat=self.nature_contrat,
                                          nbHeuresContrat=self.nb_heures_contrat, pourcentage=self.pourcentage,
                                          remunerationContrat=self.remuneration_contrat,
                                          tauxHoraires=self.taux_horaires, temoinValidite=self.temoin_validite,
                                          typeContrat=self.type_contrat, typeLienJuridique=self.type_lien_juridique)

        self.liste_coordonnees_postales = t_lcoordonneespostales(bisTer=self.bis_ter,
                                                                 bureauDistributeur=self.bureau_distributeur,
                                                                 INSEFR=self.insefr,
                                                                 codePays=self.code_pays, codePostal=self.code_postal,
                                                                 commune=self.commune,
                                                                 complementAdresse=self.complement_adresse,
                                                                 debutAdresse=self.debut_adresse,
                                                                 finAdresse=self.fin_adresse,
                                                                 natureVoie=self.nature_voie, nomVoie=self.nom_voie,
                                                                 numAdresse=self.num_adresse,
                                                                 temAdressePrincipale=self.tem_adresse_principale,
                                                                 temoinValidite=self.temoin_validite,
                                                                 typeAdresse=self.type_adresse)

        self.liste_coordonnees_bancaires = \
            t_lcoordonneesbancaires(cleCompte=self.cle_compte, codeAgence=self.code_agence,
                                    codeBanque=self.code_banque,
                                    dateDebBanque=self.date_deb_banque,
                                    dateFinBanque=self.date_fin_banque,
                                    IBAN=self.iban, libelleAgence=self.libelle_agent, modePaiement=self.mode_paiement,
                                    numCompte=self.num_compte, paysBanque=self.pays_banque, SWIFT=self.swift,
                                    temoinValidite=self.temoin_validite)

        self.liste_modalites_services = \
            t_lmodalitesservice(dateEffetModalite=self.date_effet_modalite,
                                modalite=self.modalite, temoinValidite=self.temoin_validite)

        self.liste_nationalites = \
            t_lnationalites(modeAcquisitionNationalite=self.mode_acquisition_nationalite,
                            nationalite=self.nationalite, temPrincipale=self.tem_principale)

        self.liste_numeros_telephone_fax = t_lnumerotelfax(dateDebutTel=self.date_debut_tel,
                                                           numero=self.numero, typeNumero=self.type_numero)

        self.liste_positions = t_lpostions(dateEffetPosition=self.date_effet_position,
                                           dateFinPrevue=self.date_fin_prevue,
                                           dateFinReelle=self.date_fin_reelle,
                                           position=self.position, temoinValidite=self.temoin_validite)

        self.liste_statuts = t_lstatus(dateEffetStatut=self.date_effet_statut,
                                       statut=self.statut, temoinValidite=self.temoin_validite)

        self.liste_situations_familiales = \
            t_lsituationsfamiliales(dateEffetSituFam=self.date_effet_situ_fam,
                                    situFam=self.situ_fam, temoinValidite=self.temoin_validite)
