from pydantic import BaseModel, Field
from typing import List, Optional
from wssiham.model.agent.adresse import AdresseDAP
from wssiham.model.agent.contact import EmailNumero
from wssiham.model.agent.distinction import Diplome, DistinctionHonorifique
from wssiham.model.agent.fonction import Fonction, ResponsabiliteHierarchique, FonctionElective
from wssiham.model.agent.informationIndividuel import InformationIndividuelDP, Nationalite, InformationEnfant, InformationHandicap


class InformationPersonnelle(BaseModel):
    """
    Modèle représentant les informations personnelles d'une personne
    """
    donneesPersonnelles: InformationIndividuelDP
    listeAdresses: Optional[List[AdresseDAP]]
    listeDiplomes: Optional[List[Diplome]]
    listeFonctions: Optional[List[Fonction]] = Field(alias="listFonctions")
    listeDistinctionsHonorifiques: Optional[List[DistinctionHonorifique]]
    listeEnfants: Optional[List[InformationEnfant]]
    listeFonctionsElectives: Optional[List[FonctionElective]] = Field(alias="listeFonctionsElective")
    listeHandicaps: Optional[List[InformationHandicap]]
    listeNationalites: Optional[List[Nationalite]]
    listeNumerosMails: List[EmailNumero]
    listeResponsabilitesHierarchique: Optional[List[ResponsabiliteHierarchique]] = \
        Field(alias="listeResponsabilitesHierarchi")


class InformationPersonnelleList(BaseModel):
    informationPersonnelle: InformationPersonnelle
