from pydantic import BaseModel, conint, conlist, constr, Field
from typing import Optional
from datetime import datetime


class AgentData(BaseModel):
    matricule: constr(max_length=15)
    userId: Optional[str] = Field(default=None)


class AgentDataEcheance(AgentData):
    temEtat: constr(max_length=1)
    temRetraite: constr(max_length=1)


class AgentDataModifie(AgentData):
    pass


class AgentPersonelData(AgentData):
    """
    Modèle représentant les données personnelles d'une personne.
    """
    codeCivilite: constr(max_length=1)
    dateNaissance: datetime
    nomPatronymique: Optional[str]
    nomUsuel: str
    prenom: str


class PersonalDataLA(AgentPersonelData):
    """
    Modèle représentant les données personnelles d'une personne.
    """
    codeEmploiAffectationHierarchique: Optional[str]
    codeEtablissement: str
    codePosteAffectation: Optional[str]
    codeUOAffectationHierarchique: Optional[str]
    dateDebutAffectHierarchique: Optional[datetime]
    dateFinAffectHierarchique: Optional[datetime]
    faxPro: Optional[str]
    libLongCivilite: str
    libLongEmploiAffectation: Optional[str]
    libLongPosteAffectation: Optional[str]
    libLongUOAffectationHierarchique: Optional[str]
    mailPro: Optional[str]
    numDossierHarpege: Optional[str]
    numeroInseeDefinitif: Optional[constr(max_length=15)]
    numeroInseeProv: Optional[str]
    portablePro: Optional[str]
    quotiteAffectHierarchique: Optional[str]
    telephonePro: Optional[str]


class AgentDataModifieAffec(AgentPersonelData):
    categorieStatutaire: Optional[str]
    codeEmploiAffectation: Optional[str]
    codePIP: str
    codePosteAffectation: Optional[str]
    codeUOAffectation: str
    dateDebutAffectation: datetime
    dateDepart: Optional[datetime]
    dateFinAffectation: Optional[datetime]
    libCivilite: str
    libLongCategorieStatutaire: Optional[str]
    libLongEmploiAffectation: str
    libLongPIP: str
    libLongPosteAffectation: Optional[str]
    libLongUOAffectation: str
    motifDepart: Optional[str]
    quotiteAffectation: str


class AgentDataModifieList(BaseModel):
    agent_data_modifie: conlist(AgentDataModifie, min_length=1)


class AgentDataLA(BaseModel):
    personal_data_la: conlist(PersonalDataLA, min_length=1)


class AgentDataEcheanceList(BaseModel):
    agent_data_echeance: conlist(AgentDataEcheance, min_length=1)


class AgentDataModifieAffectList(BaseModel):
    agent_data_modifie_affect: conlist(AgentDataModifieAffec, min_length=1)
