from pydantic import BaseModel, conlist


class Structure(BaseModel):
    """
    Représente une structure.

    **Champs obligatoires**

    * `codeStructure` : un code de structure, obligatoire.
    * `typeUO` : un typeUO, obligatoire.

    **Exemple**

    ```python
    information = Structure(
        codeStructure="XXXXXXXXXXXX",
        typeUO="XXXX"
    )
    ```
    """

    codeStructure: str
    typeUO: str


class StructureModifie(Structure):
    """
    Représente une structure modifiée.

    **Champs obligatoires**

    * `codeStructure` : un code de structure, obligatoire.
    * `typeUO` : un typeUO, obligatoire.

    **Exemple**

    ```python
    information = StructureModifie(
        codeStructure="XXXXXXXXXXXX",
        typeUO="XXXX"
    )
    ```
    """
    pass


class StructureEcheance(Structure):
    """
    Représente l'échéance d'une structure.

    **Champs obligatoires**

    * `codeInformation` : un code d'information, obligatoire.
    * `codeStructure` : un code de structure, obligatoire.
    * `temEtat` : un temEtat, obligatoire.
    * `typeUO` : un typeUO, obligatoire.

    **Exemple**

    ```python
    structure_echeance = StructureEcheance(
        codeInformation="XXXXX",
        codeStructure="XXXXXXXXXXXX",
        temEtat="X",
        typeUO="XXXX"
    )
    ```
    """

    codeInformation: str
    temEtat: str


class StructureModifieList(BaseModel):
    """
    Représente une liste de structure modifiée.

    **Attributs**

    * `structure_modifie` : une liste d'instances de la classe `StructureModifie`.

    **Exemple**

    ```python
    information_list = StructureModifieList(
        structure_modifie=[
            StructureModifie(
                codeStructure="XXXXXXXXXXXX",
                typeUO="XXXX"
            ),
            StructureModifie(
                codeStructure="XXXXXXXXXXXX",
                typeUO="XXX"
            )
        ]
    )
    ```
    """

    structure_modifie: conlist(StructureModifie)


class StructureEcheanceList(BaseModel):
    """
    Représente une liste d'échéance de structure.

    **Attributs**

    * `informations` : une liste d'instances de la classe `StructureEcheance`.

    **Exemple**

    ```python
    information_list = StructureEcheanceList(
        structure_echeance=[
            StructureEcheance(
                codeInformation="XXXXX",
                codeStructure="XXXXXXXXXXXX",
                temEtat="X",
                typeUO="XXXX"
            ),
            StructureEcheance(
                codeInformation="XXXX",
                codeStructure="XXXXXXXXXXXX",
                temEtat="A",
                typeUO="XXX"
            )
        ]
    )
    ```
    """

    structure_echeance: conlist(StructureEcheance)