from pydantic import BaseModel, conlist


class NomenclatureModifie(BaseModel):
    """
    Représente une liste de nomenclature modifie.

    **Champs obligatoires**

    * `codeInformation` : un code d'information, obligatoire.
    * `codeRepertoire` : un code eepertoire, obligatoire.

    **Exemple**

    ```python
    nomenclature_modifie = NomenclatureModifie(
        codeInformation="XXXXXXXXXXXX",
        codeRepertoire="XXXX"
    )
    ```
    """

    codeInformation: str
    codeRepertoire: str


class NomenclatureModifieList(BaseModel):
    """
    Représente une liste de nomenclature modifie.

    **Attributs**

    * `nomenclature_modifie` : une liste d'instances de la classe `NomenclatureModifie`.

    **Exemple**

    ```python
    nomenclature_list = NomenclatureModifieList(
        nomenclature_modifie=[
            NomenclatureModifie(
                codeInformation="XXXXXXXXXXXX",
                codeRepertoire="XXXX"
            ),
            NomenclatureModifie(
                codeInformation="XXXXXXXXXXXX",
                codeRepertoire="XXX"
            )
        ]
    )
    ```
    """

    nomenclature_modifie: conlist(NomenclatureModifie)