from pydantic import BaseModel, conlist
from typing import Optional


class NomenclaturePec(BaseModel):
    """
    Représente une nomenclature.

    **Champs obligatoires**

    * `codeNomenclature` : un code de nomenclature, obligatoire.
    * `codeRepertoire` : un code de répertoire, obligatoire.
    * `dateFinValiditeNomenclature` : une date de fin de validité de la nomenclature, obligatoire.
    * `libCourtNomenclature` : un libellé court de la nomenclature, obligatoire.
    * `libLongNomenclature` : un libellé long de la nomenclature, obligatoire.

    **Champs facultatifs**

    * `codeAdministration` : un code d'administration, facultatif.
    * `dateDebutValiditeNomenclature` : une date de début de validité de la nomenclature, facultative.
    * `ERREUR_GENERALE` : une erreur générale, facultative.
    * `REPERTOIRE_INEXISTATN` : une erreur de répertoire inexistant, facultative.

    **Exemple**

    ```python
    nomenclature = Nomenclature(
        codeNomenclature="0000",
        codeRepertoire="AAA",
        dateFinValiditeNomenclature="31/12/2999",
        libCourtNomenclature="Générique",
        libLongNomenclature="Générique",
    )
    ```
    """

    codeAdministration: Optional[str] = None

    """
    Code d'administration, facultatif.
    """

    codeNomenclature: str

    """
    Code de nomenclature, obligatoire.
    """

    codeRepertoire: str

    """
    Code de répertoire, obligatoire.
    """

    dateDebutValiditeNomenclature: Optional[str] = None

    """
    Date de début de validité de la nomenclature, facultative.
    """

    dateFinValiditeNomenclature: str

    """
    Date de fin de validité de la nomenclature, obligatoire.
    """

    ERREUR_GENERALE: Optional[str] = None

    """
    Erreur générale, facultative.
    """

    libCourtNomenclature: str

    """
    Libellé court de la nomenclature, obligatoire.
    """

    libLongNomenclature: str

    """
    Libellé long de la nomenclature, obligatoire.
    """

    REPERTOIRE_INEXISTATN: Optional[str] = None

    """
    Erreur de répertoire inexistant, facultative.
    """


class NomenclaturePecList(BaseModel):
    """Liste nomenclature Pec"""

    nomenclaturesPec: conlist(NomenclaturePec, min_length=1)
