from typing import List, Optional
from pydantic import BaseModel, conlist
from datetime import datetime


class DonneesPersonnelles(BaseModel):
    codeCivilite: str
    codeEtablissement: Optional[str]
    dateNaissance: str
    libLongCivilite: str
    matricule: str
    nomPatronymique: str
    nomUsuel: str
    prenom: str
    villeNaissance: Optional[str]


class Affectation(BaseModel):
    autre: Optional[str]
    categorieEmploiPoste: Optional[str]
    categorieFP: Optional[str]
    codeEmploiAffectation: Optional[str]
    codeEtablissement: Optional[str]
    codePosteAffectation: Optional[str]
    codeStructureNiv2: Optional[str]
    codeTypeAffectation: str
    codeTypeRattachement: str
    codeUOAffectation: str
    corpsPoste: Optional[str]
    dateDebutAffectation: str
    dateDebutPoste: Optional[str]
    dateFinAffectation: str
    emploiPoste: Optional[str]
    indiceMajore: int
    libCourtAutre: Optional[str]
    libCourtCategorieFP: Optional[str]
    libCourtCorpsPoste: Optional[str]
    libCourtPoste: Optional[str]
    libLongCodeUOAffectation: str
    libLongEmploiAffectation: Optional[str]
    libLongPoste: Optional[str]
    libLongStructureNiv2: Optional[str]
    libLongTypeAffectation: str
    libLongTypeRattachement: str
    matricule: str
    montantRemunerationMensuelle: float
    numeroMinisteriel: Optional[str]
    quotiteAffectation: str
    situationPoste: Optional[str]
    temValidite: Optional[str]
    temoinPostePermanant: Optional[str]


class Contrat(BaseModel):
    codeEtablissement: Optional[str]
    codeFrequencePaiementContrat: str
    codeModeGestionContrat: Optional[str]
    codeTypeContrat: str
    dateDebutContrat: str
    dateFinPrevueContrat: Optional[str]
    dateFinReelleContrat: str
    libLongTypeContrat: str
    matricule: str
    nbHeuresContrat: str
    numAvenantContrat: Optional[str]
    rangContrat: str
    tauxHoraireContrat: str
    temEnseignantChercheur: Optional[str]
    temValiditeContrat: Optional[str]


class PositionAdministrative(BaseModel):
    codeEtablissement: Optional[str]
    codeMotifPosition: Optional[str]
    codePositionAdmin: str
    codePositionStatutaire: str
    dateDebutPositionAdmin: str
    dateFinPrevuePositionAdmin: Optional[str]
    dateFinReellePositionAdmin: Optional[str]
    libLongMotifPosition: Optional[str]
    libLongPositionAdmin: Optional[str]
    libLongPositionStatutaire: str
    matricule: str
    temValiditePosition: str


class Statut(BaseModel):
    codeEtablissement: Optional[str]
    codeModeAccesStatut: Optional[str]
    codeStatut: str
    dateEffetStatut: str
    libLongModeAccesStatut: Optional[str]
    libLongStatut: str
    matricule: str


class Personnel(BaseModel):
    donneesPersonnelles: DonneesPersonnelles
    ERREUR_GENERALE: Optional[str]
    listeAffectations: List[Affectation]
    listeContrats: Optional[List[Contrat]]
    listePositionsAdministratives: List[PositionAdministrative]
    listeStatuts: List[Statut]


class PersonnelList(BaseModel):
    """
    Liste de personnel.
    """
    personnels: conlist(Personnel, min_length=1)
