from base64 import urlsafe_b64encode as b64e
import zlib
import pathlib
import configparser
from datetime import datetime


class Toolbox:

    @staticmethod
    def root_dir() -> str:
        """ Récupère le chemin absolu du package wssiham

        Returns: Le chemin absolu du package wssiham sous forme de chaine de caractère

        """
        return pathlib.Path(__file__).parents[1]

    @staticmethod
    def obscure(data: bytes) -> bytes:
        return b64e(zlib.compress(data, 9))

    @staticmethod
    def pathconf_ws():
        pathconfig = ""
        fileconfig = f'{__class__.root_dir()}/config.ini'
        if not pathlib.Path(fileconfig).exists():
            print("Le fichier n'existe pas")
        else:
            parserconfig = configparser.ConfigParser()
            parserconfig.read(fileconfig)
            pathconfig = parserconfig.get('APP', 'WS_CONFIG_FILE')
        return pathconfig

    @staticmethod
    def abs_pathconf_ws():
        return f'{__class__.root_dir()}/{__class__.pathconf_ws()}'

    @staticmethod
    def abs_path(path: str):
        return pathlib.Path(path).resolve()

    @staticmethod
    def environnement_name():
        fileconfig = f'{__class__.root_dir()}/config.ini'
        parserconfig = configparser.ConfigParser()
        parserconfig.read(fileconfig)
        return parserconfig.get('APP', 'ENVIRONMENT')

    @staticmethod
    def get_modification_date_file(path: pathlib.Path) -> str:
        """
        Récupère la date de modification d'un fichier sous le format suivant dd/MM/YYYY hh:mm:ss

        Args:
            path (Path): Le chemin vers le fichier

        Returns:
            str: La date de modification sous le format souhaité
        """

        # Obtenez la date de modification du fichier
        modification_date = path.stat().st_mtime

        # Convertissez la date en chaîne
        modification_date_str = datetime.fromtimestamp(modification_date).strftime("%d/%m/%Y %H:%M:%S")

        return modification_date_str
